########################################################################
# File:    reports_controller.rb                                       #
#          Hipposoft 2008                                              #
#                                                                      #
# Purpose: Generate reports describing timesheet entries in various    #
#          different ways.                                             #
#                                                                      #
# History: 09-Feb-2008 (ADH): Created.                                 #
########################################################################

class ReportsController < ApplicationController

  include TrackRecordReport
  include TrackRecordSections

  # Prepare for the 'new report' view.

  def new
    set_inclusions()

    # Make a dummy report object. In future, perhaps reports could
    # be saved into the database, owned by users.
    # [TODO]

    @report     = Report.new( @current_user, params[ :report ] )
    @task_selec = @report.tasks || []
    @task_array = @current_user.find_permitted_tasks()
    @user_array = @current_user.restricted? ?
                  [ @current_user ]         :
                  User.find_active( :all, { :order => 'name ASC' } )
  end

  # Generate a report based on a 'new report' form submission.

  def create
    set_inclusions()

    @report = Report.new( @current_user, params[ :report ] )
    @report.compile()

    respond_to do | format |
      format.html { render( { :template => 'reports/show' } ) }
      format.csv  { csv_stream_report() }
    end
  end

private

  # Set the @include_* instance variables to reflect the kind of hours being
  # included in a report.

  def set_inclusions
    @include_totals        = ( params[ :include_totals        ] == '1' )
    @include_committed     = ( params[ :include_committed     ] == '1' )
    @include_not_committed = ( params[ :include_not_committed ] == '1' )

    @include_totals = true if ( @include_totals == false && @include_committed == false && @include_not_committed == false )
  end

  # Send a report to the browser in CSV format. Requires "@report" to contain
  # a calculated report. Examines the params hash to see if additional data is
  # present which might influence the output (e.g. things to exclude, type of
  # report to send).

  def csv_stream_report()
    headings = []
    headings << ' (total)'    if ( @include_totals        )
    headings << ' (com.)'     if ( @include_committed     )
    headings << ' (not com.)' if ( @include_not_committed )

    is_task_type = params[ :user_report ].nil?

    csv_stream_data do | csv |
      if ( is_task_type )
        csv_report_by_task( csv, headings )
      else
        csv_report_by_user( csv, headings )
      end
    end
  end

  # Send a by-task CSV format report through the given CSV output stream. Pass
  # also a headings array that gives the heading suffices for each of the kinds
  # of numbers that "hours" will output based on prevailing instance variables
  # (see that function for details).

  def csv_report_by_task( csv, headings )
    # Assemble the heading row.

    file_row = [ @report.column_title ]

    @report.column_ranges.each_index do | col_index |
      partial = @report.partial_column?( col_index ) ? ' (partial)' : ''
      headings.each do | heading |
        file_row << "#{ @report.column_heading( col_index ) }#{ partial }#{ heading }"
      end
    end

    headings.each do | heading |
      file_row << "Row total#{ heading }"
    end

    if ( @report.users.empty? )
      file_row << 'Duration'
      file_row << 'Remaining (actual)'
      file_row << 'Remaining (potential)'
    end

    csv << file_row.flatten

    # Section and task list, date range breakdown.

    sections_initialise_sections()
    @report.rows.each_index do | row_index |

      row      = @report.rows[ row_index ]
      task     = @report.tasks[ row_index ]
      file_row = []

      # New section? Write out the section title and totals if so.

      if ( sections_new_section?( task ) )
        file_row << sections_section_title()

        row.cells.each_index do | col_index |
          file_row << hours( @report.sections[ sections_section_index() ].cells[ col_index ] )
        end

        file_row << hours( @report.sections[ sections_section_index() ] )

        if ( @report.users.empty? )
          file_row << '' << '' << ''
        end

        csv << file_row.flatten
        file_row = []
      end

      # Task title, data and summary information

      file_row << " -- #{ task.title }"

      row.cells.each do | cell |
        file_row << hours( cell )
      end

      file_row << hours( row )

      if ( @report.users.empty? )
        if ( task.duration == 0.0 )
          file_row << '' << '' << ''
        else
          file_row << task.duration
          file_row << task.duration - row.committed
          file_row << task.duration - row.total
        end
      end

      csv << file_row.flatten
    end

    # Column totals.

    file_row = [ 'Column total' ]

    @report.column_totals.each do | total |
      file_row << hours( total )
    end

    file_row << hours( @report )

    if ( @report.users.empty? )
      file_row << @report.total_duration
      file_row << @report.total_actual_remaining
      file_row << @report.total_potential_remaining
    end

    csv << file_row.flatten
  end

  # As "csv_report_by_task", but generates a per-user report.

  def csv_report_by_user( csv, headings )

    # Assemble the heading row.

    file_row = [ 'Name:' ]

    @report.users.each do | user |
      headings.each do | heading |
        file_row << "#{ user.name }#{ heading }"
      end
    end

    headings.each do | heading |
      file_row << "Row total#{ heading }"
    end

    csv << file_row.flatten

    # Section and task list, user breakdown.

    sections_initialise_sections()
    @report.rows.each_index do | row_index |

      row      = @report.rows[ row_index ]
      task     = @report.tasks[ row_index ]
      file_row = []

      # New section? Write out the section title and totals if so.

      if ( sections_new_section?( task ) )
        file_row << sections_section_title()

        @report.users.each_index do | user_index |
          file_row << hours( @report.sections[ sections_section_index() ].user_row_totals[ user_index ] )
        end

        file_row << hours( @report.sections[ sections_section_index() ] )

        csv << file_row.flatten
        file_row = []
      end

      # Task title, data and summary information

      file_row << " -- #{ task.title }"

      row.user_row_totals.each do | user_row_total |
        file_row << hours( user_row_total )
      end

      file_row << hours( row )
      csv << file_row.flatten
    end

    # Column totals.

    file_row = [ 'Column total' ]

    @report.user_column_totals.each do | user_column_total |
      file_row << hours( user_column_total )
    end

    file_row << hours( @report )
    csv << file_row.flatten
  end

  # Return an array with total, committed and not committed hours based
  # on the given ReportElementaryCalculator object. Items will be excluded
  # if instance variables @include_totals, @include_committed or
  # @include_not_committed, respectively, are not set to 'true'.

  def hours( calculator )
    a = []
    a << terse_hours( calculator.total         ) if ( @include_totals         )
    a << terse_hours( calculator.committed     ) if ( @include_committed      )
    a << terse_hours( calculator.not_committed ) if ( @include_not_committed  )
    return a
  end

  # Terse hours - return a given float number of hours to 2d.p. or the
  # smallest possible form (e.g. "0.0" becomes "0").

  def terse_hours( hours )
    return hours.precision( 2 ).to_s.chomp( '0' ).chomp( '0' ).chomp( '.' )
  end

  # Support CSV streamed output to the browser via FasterCSV. Requires
  # "@report" to contain a calculated report. Adapted from the 15th version
  # (20-Nov-2007 16:59) of:
  #
  #   http://wiki.rubyonrails.org/rails/pages/HowtoExportDataAsCSV

  def csv_stream_data

    label    = @report.label.downcase.gsub( ' ', '_' )
    format   = '%Y%m%d' # Compressed ISO-style
    today    = Date.current.strftime( format )
    start_at = @report.range.first.strftime( format )
    end_at   = @report.range.last.strftime( format )
    filename = "report_#{ label }_on_#{ today }_for_#{ start_at }_to_#{ end_at }.csv"

    # Required to work with IE

    if ( request.env[ 'HTTP_USER_AGENT' ] =~ /msie/i )
      headers[ 'Pragma'              ] = 'public'
      headers[ 'Cache-Control'       ] = 'maxage=3600'
      headers[ 'Content-Disposition' ] = "attachment; filename=\"#{ filename }\""
      headers[ 'Expires'             ] = '0'
    else
      headers[ 'Content-Type'        ] ||= 'text/csv'
      headers[ 'Content-Disposition' ]   = "attachment; filename=\"#{ filename }\""
    end

    render :text => Proc.new { | response, output |
      csv = FasterCSV.new( output, :row_sep => "\r\n" )
      yield csv
    }
  end
end
