########################################################################
# File:    users_helper.rb                                             #
#          Hipposoft 2008                                              #
#                                                                      #
# Purpose: Support functions for views related to User objects. See    #
#          controllers/users_controller.rb for more.                   #
#                                                                      #
# History: 03-Jan-2008 (ADH): Created.                                 #
########################################################################

module UsersHelper

  # Output HTML for the given form and user which produces a selection
  # list allowing permitted tasks to be chosen for that user. Adds a
  # warning for non-restricted users (the permitted list is ignored)
  # and handles the case where no tasks are defined. It is assumed that
  # the caller has permission to edit the user in this way.
  #
  # The including template should use appropriate container tags for the
  # output (e.g. a paragraph or table cell).

  def userhelp_permitted_task_selector( form, user )
    tasks = Task.find_active( :all )

    unless( tasks.empty? )
      Task.sort_by_augmented_title( tasks )
      output = apphelp_collection_select( form, :task_ids, tasks, :id, :augmented_title )

      unless( user.restricted? )
        output << "\n\n"
        output << "          This list is only enforced for users with a\n"
        output << "          'Normal' account type. It is included here\n"
        output << "          in case you are intending to change the account\n"
        output << "          type and want to assign tasks at the same time.<br />\n"
      end
    else
      output  = "          There are no tasks currently defined. Please\n"
      output << "          #{ link_to( 'create at least one', new_task_path() ) }."
    end

    @task_array = tasks
    @task_selec = user.tasks

    return output
  end

  # Output HTML for the given form and user which produces a selection
  # list allowing the user account type to be chosen. It is assumed that
  # the caller has permission to edit the user in this way. Sets the
  # instance variable "@task_array" to the list of tasks used and
  # "@task_selec" to the user's current default task list, in passing.
  #
  # The including template should use appropriate container tags for the
  # output (e.g. a paragraph or table cell).

  def userhelp_user_type_selector( form, user )
    if ( @current_user.id == user.id and user.admin? )

      # You can't change your own account type if you are an administrator.
      # This is an easy way to prevent all administrators losing privilege,
      # leaving no administrator in the system.

      output  = "          Administrators cannot change account type. If you\n"
      output << "          want to stop being an administrator, first assign\n"
      output << "          administrative rights to another account, then\n"
      output << "          have that user change your account type for you."

    elsif ( not @current_user.admin? and user.admin? )

      # If the editing account is administrative but the user is not,
      # then the account type can't be changed (managers can only assign
      # privilege up to manager).

      output  = "          Administrator. Only another administrator can change\n"
      output << "          this user's account type.\n"

    else

      # OK, this is your account and you're not an administrator, or
      # it is someone else's account and they're not an administrator.

      if ( @current_user.admin? )
        warn    = false
        options = [
                    [ 'Normal',        'Normal'  ],
                    [ 'Manager',       'Manager' ],
                    [ 'Administrator', 'Admin'   ]
                  ]
      else
        warn    = true
        options = [
                    [ 'Normal',  'Normal'  ],
                    [ 'Manager', 'Manager' ]
                  ]
      end

      output = apphelp_select( form, :user_type, options, false )

      if ( warn and @current_user.id == user.id )
        output << "\n\n"
        output << "          <p>\n"
        output << "            Note that if you revoke your own account privileges,\n"
        output << "            you will need the help of another manager or an\n"
        output << "            administrator if you want to restore them later.\n"
        output << "          </p>"
      end
    end

    return output
  end

  # Output HTML for the given form and user which produces a selection
  # list allowing the user's 'active' flag to be changed. It is assumed
  # that the caller has permission to edit the user in this way.

  def userhelp_active_selector( form, user )
    return apphelp_select(
      form,
      :active,
      [
        [ 'Active',      true  ],
        [ 'Deactivated', false ]
      ],
      false
    )
  end

  # Output HTML for the given control panel form (fields_for...) and user
  # which lets the user choose the default tasks for new timesheets. Sets
  # the instance variable "@task_array" to the list of tasks used and
  # "@task_selec" to the user's current default task list, in passing.
  #
  # The including template should use appropriate container tags for the
  # output (e.g. a paragraph or table cell).

  def userhelp_default_task_selector( cp, user )
    tasks = user.nil? ? [] : user.find_permitted_tasks( nil )

    unless( tasks.empty? )
      Task.sort_by_augmented_title( tasks )
      output  = apphelp_collection_select( cp, :task_ids, tasks, :id, :augmented_title )
    else
      output  = "          This account does not have permission to view\n"
      output << "          any tasks.\n"
      
      unless( user.restricted? )
        output << "\n\n"
        output << "          <p>\n"
        output << "            To enable this section, please assign tasks to\n"
        output << "            the user account with the security settings and\n"
        output << "            save your changes.\n" 
        output << "          </p>"
      end
    end

    @task_array = tasks
    @task_selec = user.control_panel.nil? ? [] : user.control_panel.tasks

    return output
  end

  # Output HTML for the given user's control panel editing section within a
  # wider user editing form, which lets the user choose the default project
  # associated with new tasks.
  #
  # The including template should use appropriate container tags for the
  # output (e.g. a paragraph or table cell).

  def userhelp_default_project_selector( user )
    projects = Project.find_active( :all, :order => Project::DEFAULT_SORT_ORDER )

    unless( projects.empty? )
      output = apphelp_project_selector(
        'control_panel_project_id',
        'control_panel[project_id]',
        user.control_panel.nil? ? nil : user.control_panel.project_id,
        true
      )

      if( user.restricted? )
        output << "\n\n"
        output << "          <p>\n"
        output << "            This list is only used for users with a\n"
        output << "            privileged account type because normal users\n"
        output << "            cannot create new tasks.\n"
        output << "          </p>"
      end
    else
      output  = "          There are no projects currently defined. Please\n"
      output << "          #{ link_to( 'create at least one', new_project_path() ) }."
    end

    return output
  end

  # Output HTML for the given control panel form (fields_for...) and user
  # which lets the user choose the default customer associated with new
  # projects.
  #
  # The including template should use appropriate container tags for the
  # output (e.g. a paragraph or table cell).

  def userhelp_default_customer_selector( cp, user )
    customers = Customer.find_active( :all, :order => Customer::DEFAULT_SORT_ORDER )

    unless( customers.empty? )
      output = apphelp_collection_select( cp, :customer_id, customers, :id, :title, false )

      if( user.restricted? )
        output << "\n\n"
        output << "          <p>\n"
        output << "            This list is only used for users with a\n"
        output << "            privileged account type because normal users\n"
        output << "            cannot create new tasks.\n"
        output << "          </p>"
      end
    else
      output  = "          There are no customers currently defined. Please\n"
      output << "          #{ link_to( 'create at least one', new_customer_path() ) }."
    end

    return output
  end

  # List view helper - format the given user's e-mail address as a link; do
  # so using auto_link so we don't have to worry about malformed addresses.
  # That's the main reason for doing this in a helper rather than manually,
  # within the model.

  def userhelp_email( user )
    return auto_link( h( user.email ) )
  end

  # As above, but for the user's identity URL

  def userhelp_identity_url( user )
    return auto_link( h( user.identity_url ) )
  end

  # Return list actions appropriate for the given user

  def userhelp_actions( user )
    actions = [ 'edit' ]
    actions.push( 'delete' ) unless user.admin?
    actions.push( 'show' )
    return actions
  end
end
